@@ -81,6 +81,9 @@ exclude_pars_internal.brmsterms <- function(x, save_all_pars, save_mevars, ...)
     for (dp in names(x$dpars)) {
       c(out) <- exclude_pars_internal(x$dpars[[dp]], ...)
     }
+    for (nlp in names(x$nlpars)) {
+      c(out) <- exclude_pars_internal(x$nlpars[[nlp]], ...)
+    }
   }
   if (!save_mevars && is.formula(x$adforms$mi)) {
     c(out) <- paste0("Yl", p)
@@ -88,21 +91,13 @@ exclude_pars_internal.brmsterms <- function(x, save_all_pars, save_mevars, ...)
   out
 }
 
-#' @export
-exclude_pars_internal.btnl <- function(x, ...) {
-  out <- NULL
-  for (nlp in names(x$nlpars)) {
-    c(out) <- exclude_pars_internal(x$nlpars[[nlp]], ...)
-  }
-  out
-}
-
 #' @export
 exclude_pars_internal.btl <- function(x, data, ...) {
   p <- usc(combine_prefix(x))
   out <- c(
     paste0("temp", p, "_Intercept"),
-    paste0(c("hs_local", "hs_global", "zb"), p)
+    paste0(c("hs_local", "hs_global", "zb"), p),
+    paste0(c("hs_localsp", "zbsp"), p)
   )
   smef <- tidy_smef(x, data)
   for (i in seq_len(nrow(smef))) {
