@@ -113,6 +113,12 @@ extract_draws.brmsterms <- function(x, samples, sdata, ...) {
       draws$dpars[[dp]] <- as.vector(get_samples(samples, dp_regex))
     }
   }
+  draws$nlpars <- named_list(names(x$nlpars))
+  for (nlp in names(x$nlpars)) {
+    draws$nlpars[[nlp]] <- extract_draws(
+      x$nlpars[[nlp]], samples = samples, sdata = sdata, ...
+    )
+  }
   if (is.mixfamily(x$family)) {
     families <- family_names(x$family)
     thetas <- paste0("theta", seq_along(families))
@@ -145,19 +151,10 @@ extract_draws.brmsterms <- function(x, samples, sdata, ...) {
 extract_draws.btnl <- function(x, samples, sdata, ...) {
   draws <- list(
     f = x$family, nlform = x$formula[[2]],
-    nsamples = nrow(samples), nobs = sdata$N
+    nsamples = nrow(samples), nobs = sdata$N,
+    used_nlpars = x$used_nlpars
   )
   class(draws) <- "bdrawsnl"
-  if (is_nlpar(x)) {
-    draws$nlpar <- x$nlpar
-    return(draws)
-  }
-  nlpars <- names(x$nlpars)
-  for (nlp in nlpars) {
-    draws$nlpars[[nlp]] <- extract_draws(
-      x$nlpars[[nlp]], samples = samples, sdata = sdata, ...
-    )
-  }
   p <- usc(combine_prefix(x))
   C <- sdata[[paste0("C", p)]]
   stopifnot(is.matrix(C))
@@ -395,37 +392,76 @@ extract_draws_gp <- function(bterms, samples, sdata, data,
   }
   draws <- named_list(gpef$label)
   for (i in seq_along(draws)) {
-    gp <- list()
-    gp_names <- paste0(gpef$label[i], usc(gpef$bylevels[[i]]))
-    sdgp <- paste0("^sdgp", p, "_", gp_names)
-    gp$sdgp <- get_samples(samples, sdgp)
-    lscale <- paste0("^lscale", p, "_", gp_names)
-    gp$lscale <- get_samples(samples, lscale)
-    zgp <- paste0("^zgp", p, "_", gpef$label[i], "\\[")
-    gp$zgp <- get_samples(samples, zgp)
-    gp$bynum <- sdata[[paste0("Cgp", p, "_", i)]]
-    Jgp_pos <- grepl(paste0("^Jgp", p, "_", i), names(sdata))
-    if (new) {
-      gp$x <- old_sdata[[paste0("Xgp", p, "_", i)]]
-      gp$x_new <- sdata[[paste0("Xgp", p, "_", i)]]
-      gp$Jgp <- old_sdata[Jgp_pos]
-      gp$Jgp_new <- sdata[Jgp_pos]
-      # computing GPs for new data requires the old GP terms
-      gp$yL <- .predictor_gp(
-        x = gp[["x"]], sdgp = gp[["sdgp"]],
-        lscale = gp[["lscale"]], zgp = gp[["zgp"]], 
-        Jgp = gp[["Jgp"]]
-      )
+    lvls <- gpef$bylevels[[i]]
+    if (length(lvls)) {
+      gp <- named_list(lvls)
+      for (j in seq_along(lvls)) {
+        gp[[lvls[j]]] <- .extract_draws_gp(
+          gpef, samples = samples, sdata = sdata,
+          old_sdata = old_sdata, nug = nug, new = new,
+          byj = j, p = p, i = i
+        )
+      }
+      attr(gp, "byfac") <- TRUE
     } else {
-      gp$x <- sdata[[paste0("Xgp", p, "_", i)]]
-      gp$Jgp <- sdata[Jgp_pos]
+      gp <- .extract_draws_gp(
+        gpef, samples = samples, sdata = sdata,
+        old_sdata = old_sdata, nug = nug, new = new, 
+        p = p, i = i
+      )
     }
-    gp$nug <- nug
     draws[[i]] <- gp
   }
   draws
 }
 
+.extract_draws_gp <- function(gpef, samples, sdata, old_sdata,
+                              nug, new, p, i, byj = NULL) {
+  # extract draws for Gaussian processes
+  # Args:
+  #   gpef: output of tidy_gpef
+  #   p: prefix created by combine_prefix()
+  #   i: indiex of the Gaussian process
+  #   byj: index for the level of a categorical 'by' vaiable
+  # Return:
+  #   a list to be evaluated by .predictor_gp()
+  if (is.null(byj)) {
+    lvl <- ""
+  } else {
+    lvl <- gpef$bylevels[[i]][byj]
+  }
+  j <- usc(byj)
+  gp <- list()
+  gp_name <- escape_all(paste0(gpef$label[i], lvl))
+  sdgp <- paste0("^sdgp", p, "_", gp_name, "$")
+  gp$sdgp <- as.numeric(get_samples(samples, sdgp))
+  lscale <- paste0("^lscale", p, "_", gp_name, "$")
+  gp$lscale <- as.numeric(get_samples(samples, lscale))
+  zgp_regex <- paste0("^zgp", p, "_", gp_name, "\\[")
+  gp$zgp <- get_samples(samples, zgp_regex)
+  Xgp_name <- paste0("Xgp", p, "_", i, j)
+  Igp_name <- paste0("Igp", p, "_", i, j)
+  Jgp_name <- paste0("Jgp", p, "_", i, j)
+  if (new) {
+    gp$x <- old_sdata[[Xgp_name]]
+    gp$nug <- 1e-11
+    # computing GPs for new data requires the old GP terms
+    gp$yL <- .predictor_gp(gp)
+    gp$x_new <- sdata[[Xgp_name]]
+    gp$Igp <- sdata[[Igp_name]]
+  } else {
+    gp$x <- sdata[[Xgp_name]]
+    gp$Igp <- sdata[[Igp_name]]
+  }
+  gp$Jgp <- sdata[[Jgp_name]]
+  if (is.null(byj)) {
+    # possible continuous 'by' variable
+    gp$bynum <- sdata[[paste0("Cgp", p, "_", i)]]
+  }
+  gp$nug <- nug
+  gp
+}
+
 extract_draws_re <- function(bterms, samples, sdata, data, ranef, old_ranef, 
                              sample_new_levels = "uncertainty", ...) {
   # extract draws of group-level effects
