@@ -761,6 +761,11 @@ mixture <- function(..., flist = NULL, nmix = 1, order = NULL) {
   if (any(is_ordinal) && any(!is_ordinal)) {
     stop2("Cannot mix ordinal and non-ordinal families.")
   }
+  for (fam in family$mix) {
+    if (is.customfamily(fam) && "theta" %in% fam$dpars) {
+      stop2("Parameter name 'theta' is reserved in mixture models.")
+    }
+  }
   if (is.null(order)) {
     if (any(is_ordinal)) {
       family$order <- "none"
@@ -1234,6 +1239,12 @@ allow_cs <- function(family) {
   "cs" %in% family_info(family, "specials")
 }
 
+has_logscale <- function(family) {
+  # indicate if the response is modeled on the log-scale
+  # even if formally the link function is not 'log'
+  "logscale" %in% family_info(family, "specials")
+}
+
 has_trials <- function(family) {
   # indicate if family makes use of argument trials
   "trials" %in% family_info(family, "ad") &&
