@@ -74,7 +74,7 @@ test_that("all S3 methods have reasonable ouputs", {
   expect_equal(dim(R2), c(nsamples(fit1), 1))
   R2 <- bayes_R2(fit2, newdata = model.frame(fit2)[1:5, ])
   expect_equal(dim(R2), c(1, 4))
-  expect_error(bayes_R2(fit4), "Residuals are not defined for ordinal")
+  expect_error(bayes_R2(fit4), "'bayes_R2' is not defined for ordinal")
   R2 <- bayes_R2(fit6)
   expect_equal(dim(R2), c(2, 4))
   
@@ -163,7 +163,7 @@ test_that("all S3 methods have reasonable ouputs", {
   
   # hypothesis
   hyp <- hypothesis(fit1, c("Intercept > Trt1", "Trt1:Age = -1"))
-  expect_equal(dim(hyp$hypothesis), c(2, 7))
+  expect_equal(dim(hyp$hypothesis), c(2, 8))
   expect_output(print(hyp), "(Intercept)-(Trt1) > 0", fixed = TRUE)
   expect_true(is(plot(hyp, plot = FALSE)[[1]], "ggplot"))
   
@@ -173,7 +173,7 @@ test_that("all S3 methods have reasonable ouputs", {
   expect_true(is(plot(hyp, ignore_prior = TRUE, plot = FALSE)[[1]], "ggplot"))
   
   hyp <- hypothesis(fit1, "0 > r_visit[4,Intercept]", class = "", alpha = 0.01)
-  expect_equal(dim(hyp$hypothesis), c(1, 7))
+  expect_equal(dim(hyp$hypothesis), c(1, 8))
   expect_output(print(hyp, chars = NULL), "r_visit[4,Intercept]", fixed = TRUE)
   expect_output(print(hyp), "99%-CI", fixed = TRUE)
   
@@ -181,7 +181,7 @@ test_that("all S3 methods have reasonable ouputs", {
     fit1, c("Intercept = 0", "Intercept + exp(Trt1) = 0"),
     group = "visit", scope = "coef"
   )
-  expect_equal(dim(hyp$hypothesis), c(8, 8))
+  expect_equal(dim(hyp$hypothesis), c(8, 9))
   expect_equal(hyp$hypothesis$Group[1], "1")
   
   expect_error(hypothesis(fit1, "Intercept > x"), fixed = TRUE,
@@ -197,9 +197,9 @@ test_that("all S3 methods have reasonable ouputs", {
   
   # test hypothesis.default method
   hyp <- hypothesis(as.data.frame(fit3), "bsp_meAgeAgeSD > sigma")
-  expect_equal(dim(hyp$hypothesis), c(1, 7))
+  expect_equal(dim(hyp$hypothesis), c(1, 8))
   hyp <- hypothesis(fit3$fit, "bsp_meAgeAgeSD > sigma")
-  expect_equal(dim(hyp$hypothesis), c(1, 7))
+  expect_equal(dim(hyp$hypothesis), c(1, 8))
   
   # omit launch_shiny
   
@@ -328,7 +328,7 @@ test_that("all S3 methods have reasonable ouputs", {
                  "r_patient__a[1,Intercept]", "r_patient__b[4,Intercept]",
                  "prior_b_a"))
   expect_true(all(
-    c("lscale_volume_gpAgeTrt_0", "lscale_volume_gpAgeTrt_1") %in% 
+    c("lscale_volume_gpAgeTrt0", "lscale_volume_gpAgeTrt1") %in% 
       parnames(fit6)
   ))
   
@@ -663,6 +663,13 @@ test_that("all S3 methods have reasonable ouputs", {
   # ------ tests skipped on CRAN ------ #
   skip_on_cran()
   
+  # loo_R2
+  R2 <- SW(loo_R2(fit2))
+  expect_equal(length(R2), 1)
+  
+  R2 <- SW(loo_R2(fit6))
+  expect_equal(length(R2), 2)
+  
   # loo
   loo1 <- SW(LOO(fit1, cores = 1))
   expect_true(is.numeric(loo1$estimates))
